#ifndef __geom23_h /* Copyright Bill.Triggs@inrialpes.fr */
#define __geom23_h "$ Fri Jul 21 12:17:56 2000 $"
/** NO WARRANTY. Use & distribute freely. Fixes/feedback welcome. **/
/*------------------------------------------------------------------------
  2D and 3D geometry routines
 ------------------------------------------------------------------------*/

/* Indices for 2D and 3D vectors */
enum {X=0,Y=1,Z=2,W=3};

#ifndef SQ
# define SQ(x) ((x)*(x))      
#endif

/* 3x3 matrix times 3-vector */
# define m33v3(O,M,I) \
((O)[X]=(M)[X][X]*(I)[X]+(M)[X][Y]*(I)[Y]+(M)[X][Z]*(I)[Z],\
 (O)[Y]=(M)[Y][X]*(I)[X]+(M)[Y][Y]*(I)[Y]+(M)[Y][Z]*(I)[Z],\
 (O)[Z]=(M)[Z][X]*(I)[X]+(M)[Z][Y]*(I)[Y]+(M)[Z][Z]*(I)[Z])

/* 3x4 image projection of a 3-component affine point. */
#define p34v3(O,P,I) \
((O)[X]=(P)[X][X]*(I)[X]+(P)[X][Y]*(I)[Y]+(P)[X][Z]*(I)[Z]+(P)[X][W],\
 (O)[Y]=(P)[Y][X]*(I)[X]+(P)[Y][Y]*(I)[Y]+(P)[Y][Z]*(I)[Z]+(P)[Y][W],\
 (O)[Z]=(P)[Z][X]*(I)[X]+(P)[Z][Y]*(I)[Y]+(P)[Z][Z]*(I)[Z]+(P)[Z][W])

/* squared norms of vectors */
#define v3sq(v) (SQ((v)[X])+SQ((v)[Y])+SQ((v)[Z]))
#define v4sq(v) (SQ((v)[X])+SQ((v)[Y])+SQ((v)[Z])+SQ((v)[W]))

/* Cross-product of 3-vectors.*/
# define v3cross(O,A,B) \
((O)[X]=(A)[Y]*(B)[Z]-(A)[Z]*(B)[Y],\
 (O)[Y]=(A)[Z]*(B)[X]-(A)[X]*(B)[Z],\
 (O)[Z]=(A)[X]*(B)[Y]-(A)[Y]*(B)[X])

/* Copy for fixed-size arrays */
#ifndef acopy
# define acopy(typ,a,b,siz) do{struct __t{typ x[siz];};\
			     *(struct __t*)(a)=*(struct __t*)(b);}while(0)
#endif

extern double randn(void);
extern double (*quat_to_rot(double R[3][3], double q[4], int normalize))[3];
extern double *quat_mul(double qout[4], double qa[4], double qb[4]);
extern double *quat_left_div(double qout[4], double qa[4], double qb[4]);
extern double* rot_to_quat(double q[4], double R[3][3]);


/*------------------------------------------------------------------------
  Linear algebra stuff.
 ------------------------------------------------------------------------*/
/* Temporary memory allocation for local arrays and workspace,
 * (i.e. memory not passed back from allocating routine). This can be
 * alloca if you like.
 */
#ifndef TMPALLOC
# include <stdlib.h>
# define TMPALLOC(type,n) ((type*)malloc((n)*sizeof(type)))
# define TMPFREE(ptr) (free(ptr))
#endif

/* How to call LAPACK routines.
 */
#ifndef LAPACK
# define LAPACK(func) func##_
#endif


#endif /*__geom23_h */
